<?php

namespace App\HttpController\Street;

use App\Key\K;

use App\Model\Street\EarthDataTableList;
use App\Model\Street\EarthDownloadList;
use App\Model\Street\EarthErrorList;
use App\Model\Street\StreetDownloadList;
use App\Model\Street\StreetErrorList;
use App\Tools\P;
use App\Tools\System;
use EasySwoole\Component\TableManager;
use EasySwoole\Mysqli\QueryBuilder;
use EasySwoole\ORM\DbManager;

//这个类主要是下载谷歌地球的数据使用的
class Earth extends Base
{
    public function cc()
    {

        $r = P::getRedis();
//        $r->del(K::GOOGLE_EARTH_CACHE_LIST);
        $all = $r->hGetAll(K::GOOGLE_EARTH_CACHE_LIST);
        return $this->jsonSuccess($all);
    }

    public function index()
    {
        $outDate = [];
        $systemTable = TableManager::getInstance()->get(K::TABLE_SYSTEM_CONFIG);
        //如果有正在请求的，那么返回[];
        if ($systemTable->get(K::GOOGLE_EARTH_IS_GETTING_ID)) return $this->jsonSuccess(['ids' => [], 'has_run' => 1]);
        $systemTable->set(K::GOOGLE_EARTH_IS_GETTING_ID, ['data' => 1, 'number' => 1]);
        $redis = P::getRedis();
        $cacheData = $redis->hGetAll(K::GOOGLE_EARTH_CACHE_LIST);
        //注意，这个ID，不能给的太大了，要不然子服务器无法处理大量的数据
        $id_limit = 2;
        $workInfo = $redis->get(K::GOOGLE_EARTH_CURRENT_WORK_INFO);
        foreach ($cacheData as $v) {
            if (time() - $v['time'] > 1500) {
                //超时10分钟，那么认为子服务器没有下载完成，注意，里面的数据也是一个数组，要和下面的对应起来
                $outDate[] = ['id' => $v['id']];
                //将这一部分的数据重新放到缓存中，进行处理
                $v['time'] = time();
                $redis->hSet(K::GOOGLE_EARTH_CACHE_LIST, $v['id'], $v);
                if (count($outDate) >= $id_limit) break;
            }
        }
        $cacheLength = count($cacheData);
        if ($outDate) {
            echo 'use cache ', PHP_EOL;
            //如果有缓存的数据 ，先使用缓存的数据
            $systemTable->del(K::GOOGLE_EARTH_IS_GETTING_ID);
            return $this->jsonSuccess(['ids' => $outDate, 'work_info' => $workInfo]);
        }
        if ($cacheLength > 1000) {
            $systemTable->del(K::GOOGLE_EARTH_IS_GETTING_ID);
            //如果缓存中已经存在太多的数据，不会返回
            System::log('too many cache ids wait...');
            return $this->jsonSuccess(['ids' => []]);
        }
        $nextTable = EarthDataTableList::getNextTableName();
        if (!$nextTable) {
            $systemTable->del(K::GOOGLE_EARTH_IS_GETTING_ID);
            System::log('no work to do');
            if (!$cacheLength) {
                //此时，要返回 done 参数 ，表示服务器已经没有更多的数据了，要将最后一个ID上传上来
                return $this->jsonSuccess(['ids' => [], 'done' => 1]);
            } else {
                return $this->jsonSuccess(['ids' => []]);
            }
        }
        $redis->setEx(K::GOOGLE_EARTH_CURRENT_WORK_INFO, 86400 * 360, $nextTable);
        $queryBuilder = new QueryBuilder();
        //        / 支持参数绑定 第二个参数非必传
        // $queryBuild->raw("select * from test where name = 'siam'");
//        if (!$nextTable['next_id']) {
//            $queryBuilder->raw("select table_id,id from " . $nextTable['name'] . "  order by table_id ASC limit " . $id_limit);
//        } else {
//            $queryBuilder->raw("select table_id,id from " . $nextTable['name'] . "  where table_id > ? order by table_id ASC limit " . $id_limit, [$nextTable['next_id']]);
//        }

        $queryBuilder->raw("select id from " . $nextTable['name'] . "  where status=0 limit " . $id_limit);

        // 第二个参数 raw  指定true，表示执行原生sql
        // 第三个参数 connectionName 指定使用的连接名，默认 default
        $ids = DbManager::getInstance()->query($queryBuilder, true, K::DB_CONNECT_STREET);
        //这里要注意，返回的结果集是在 result 下标中
        $ids = $ids->toArray();
        $ids = $ids['result'];
        foreach ($ids as $id) {
//            $nextTable['next_id'] = $id['table_id'];
            $queryBuilder->raw("update " . $nextTable['name'] . "  set status=1 where id=? ",[$id['id']]);
            DbManager::getInstance()->query($queryBuilder, true, K::DB_CONNECT_STREET);
            //设置time 参数的主要作用是用来判断超时使用
            $redis->hSet(K::GOOGLE_EARTH_CACHE_LIST, $id['id'], ['id' => $id['id'], 'time' => time()]);
        }
        System::log('current google earth worker next_id : ' . $nextTable['name'] . '-' . $nextTable['next_id']);
        $dataModel = new EarthDataTableList();
        $next = '';
        $done = 0;
        if (count($ids) < $id_limit) {
            //如果已经少于指定数量的ID，那么，要根据情况来重新生成数据
            if (!count($ids)) {
                $cacheData = $redis->hGetAll(K::GOOGLE_EARTH_CACHE_LIST);
                if (!$cacheData) {
                    System::log('update next_id to empty new table');
                    $next = 1;
                    $done = 1;
                } else {
                    System::log('wait some server to download ok');
                }
            }
        }
        $dataModel->where(['id' => $nextTable['id']])->update(['next_id' => $nextTable['next_id'], 'done_time' => time(), 'done' => $done]);
        $systemTable->del(K::GOOGLE_EARTH_IS_GETTING_ID);
        return $this->jsonSuccess(['ids' => $ids, 'next' => $next, 'work_info' => $nextTable]);
    }

    public function confirm()
    {
        //客户端确认数据已经下载完成，那么将缓存中的数据要删除掉
        $data = $this->getRequestData();
        $redis = P::getRedis();
        $workInfo = $redis->get(K::GOOGLE_EARTH_CURRENT_WORK_INFO);
        DbManager::getInstance()->startTransaction('street');
        $queryBuilder = new QueryBuilder();
        //这里返回的结果也是有下标的，要对应起来
        foreach ($data['ids'] as $v) {
            if ($workInfo['id'] && $data['work_info']['id'] && $workInfo['id'] == $data['work_info']['id']) {
                $queryBuilder->raw("update " . $workInfo['name'] . ' set status=2 where id =?', [$v['id']]);
                DbManager::getInstance()->query($queryBuilder, true, K::DB_CONNECT_STREET);
                $redis->hDel(K::GOOGLE_EARTH_CACHE_LIST, $v['id']);
            }
        }
        DbManager::getInstance()->commit('street');
        System::log('google earth confirm ids size : ' . count($data['ids']));
        return $this->jsonSuccess([]);
    }

    //子服务器下载数据，需要把生成的metadata 提交上来
    public function meta()
    {
        switch ($this->request()->getMethod()) {
            case 'POST':
                $redis = P::getRedis();
                $workInfo = $redis->get(K::GOOGLE_EARTH_CURRENT_WORK_INFO);
                DbManager::getInstance()->startTransaction('street');
                $data = $this->getRequestData();
                $queryBuilder = new QueryBuilder();
                foreach ($data['ids'] as $v) {
                    if ($workInfo['id'] && $data['work_info']['id'] && $workInfo['id'] == $data['work_info']['id']) {
                        //如果提交上来的任务不是当前的任务，那么要跳过
                        $queryBuilder->raw("select id from " . $workInfo['name'] . ' where id =?', [$v]);
                        $has = DbManager::getInstance()->query($queryBuilder, true, K::DB_CONNECT_STREET);
                        $has=$has->toArray();
                        $has = $has['result'];
                        if (!$has) {
                            $queryBuilder->raw("insert into " . $workInfo['name'] . ' (`id`) values(?)', [$v]);
                            DbManager::getInstance()->query($queryBuilder, true, K::DB_CONNECT_STREET);
                        }
                    }
                }
                DbManager::getInstance()->commit('street');
                break;
        }
    }

    //子服务器将下载好的数据提交到这里来
    public function db()
    {
        $data = $this->getRequestData();
        $model = new EarthDownloadList();
        DbManager::getInstance()->startTransaction('street');
        $del = [];
        foreach ($data['dbs'] as $v) {
            $has = $model->where(['id' => $v])->get();
            if ($has) {
                if ($has['is_done']) {
                    $model->where(['id' => $v])->update(['is_delete' => 1]);
                    $del[] = $v;
                }
            } else {
                $obj = $model::create(['id' => $v, 'ip' => $this->ip]);
                $obj->save();
            }
        }
        DbManager::getInstance()->commit('street');
        return $this->jsonSuccess(['del' => $del]);
    }

    //这个是获取当前有多少个IP服务器
    public function server()
    {
        $model = new EarthDownloadList();
        switch ($this->request()->getMethod()) {
            case 'GET':
                $ips = $model->getIps();
                return $this->jsonSuccess($ips);
                break;
        }
    }

    //在本地的服务器将数据下载完成了，那么要通知主服务器，这个文件下载完成，方便下载服务器删除数据
    public function download()
    {
        $model = new EarthDownloadList();
        switch ($this->request()->getMethod()) {
            case 'POST':
            case 'GET':
                //这里的数据需要按IP来分类
                $data = $this->getRequestData();
                if ($data['ip']) {
                    $ip = [$data['ip']];
                } else {
                    $ip = $model->getIps();
                }
                $result = [];
                foreach ($ip as $v) {
                    $result[$v] = $model->order('id', 'ASC')->where(['is_done' => 0, 'ip' => $v])->limit(20)->all();
                    $result[$v] = array_map(function ($v) {
                        return $v->toArray();
                    }, $result[$v]);
                }
                return $this->jsonSuccess($result);
                break;
            case 'PUT':
                $data = $this->getRequestData();
                if (!$data['id']) return $this->jsonError('no id');
                $result = $model->groupSave($data['id'], 'edit_download', ['is_done' => 1, 'download_time' => time()]);
                if (!$result) return $this->jsonError($model->getError());
                return $this->jsonSuccess($result);
                break;
        }
    }

    public function error()
    {
        $data = $this->getRequestData();
        $model = new EarthErrorList();
        DbManager::getInstance()->startTransaction('street');
        foreach ($data as $v) {
            $data_str = is_array($v['data']) ? json_encode($v['data']) : $v['data'];
            $obj = $model::create(['ip' => $this->ip, 'data' => $data_str, 'create_time_str' => date('Y-m-d H:i:s')]);
            $obj->save();
        }
        DbManager::getInstance()->commit('street');
        return $this->jsonSuccess(['ok']);
    }
}
