<?php

namespace App\Tools;

use App\Key\K;
use EasySwoole\Component\TableManager;
use EasySwoole\Component\Timer;
use EasySwoole\EasySwoole\ServerManager;
use SQLite3;
use Swlib\Saber;
use Swlib\SaberGM;
use Swoole\Coroutine;

class GoogleEarthDownloader
{
    //注意，这里的两个URL前面都默认带了0 ，主要是为了和Cesium进行匹配。防止出错
    public $version = 0;
    public $key = '';
    private $tileUrl = 'http://kh.google.com/flatfile?f1-0{xyz}-i.{v}';
    private $metaUrl = 'http://kh.google.com/flatfile?q2-0{xyz}-q.{v}';
    /**
     * @var SQLite3
     */
    private $sqlite;

    private $sqliteConfig = [
        'ok' => false,
        'name' => '',
        'sum' => 0,
        'dir' => '',
        'limit' => 50000 * 3,//超过这个值，会重新生成新的sqlite文件
    ];
    private $workerId;
    private $workerHash;
    private $bounds = [];
    private $speed = [
        'sum' => 0,//
        'total' => 0,//
        'ok' => 0,
        'has' => 0,
        'commit' => 0,
        'minute' => '',
        'ids' => '',
    ];

    private $thread = [
        'sum' => 10,//开启4个协程来下载数据
        'status' => [],//记录所有协程的状态，如果已经没有了数据，那么，重新拉新的数据
        'done' => 0,//已经有多少个协程下载完成，方便拉新的任务
    ];
    private $work = [
        'url' => [],
        'bounds'=>[],
        'code' => 200,
        'start_zoom' => 0,
        'end_zoom' => 0,
        'work_info'=>[],
        'status' => 0,//这个值是使用来记录，如果服务器没有返回要下载的列表，那么要将最后一个文件上传
    ];
    public $metaDataZoom = [0, 2, 6, 10, 14, 18];
    private $error = [
        'sum' => 0,
        'list' => [],
        'limit' => 10000
    ];

    public function __construct()
    {
        $this->sqliteConfig['dir'] = System::getGoogleEarthDir();
        if (!is_dir($this->sqliteConfig['dir'])) mkdir($this->sqliteConfig['dir']);
    }

    //如果传入了name 表示设置，否则表示获取数据
    public function workerDbName($name = '')
    {
        $configTable = TableManager::getInstance()->get(K::TABLE_SYSTEM_CONFIG);
        $currentDownloadDbs = $configTable->get(K::CURRENT_GOOGLE_EARTH_DOWNLOAD_ID);
        if (!$currentDownloadDbs) {
            $currentDownloadDbs = ['number' => [], 'data' => []];
        } else {
            $currentDownloadDbs['data'] = json_decode($currentDownloadDbs['data'], true);
        }
        if ($name) {
            $currentDownloadDbs['data'][$this->workerHash] = $this->sqliteConfig['name'];
            $currentDownloadDbs['data'] = json_encode($currentDownloadDbs['data']);
            $configTable->set(K::CURRENT_MAP_DOWNLOAD_ID, $currentDownloadDbs);
        } else {
            return $currentDownloadDbs['data'];
        }
    }

    public function initSqlite()
    {
        $dbDir = $this->sqliteConfig['dir'];
        $this->sqliteConfig['name'] = 'earth=' . date('Y-m-d-H-i-s') . '=' . strtolower(uniqid()) . '=' . $this->workerId . '.db';
        $this->sqliteConfig['file_dir'] = $dbDir . '/' . $this->sqliteConfig['name'];
        $this->sqlite = new SQLite3($dbDir . '/' . $this->sqliteConfig['name']);
        $this->sqlite->query('CREATE TABLE "tile" (
            "id"	TEXT UNIQUE,
            "data"	BLOB,
            PRIMARY KEY("id")
        )');
        //将当前的名称放到table 中，方便在多线程下载的时候进行判断。
        $this->workerDbName($this->sqliteConfig['name']);
        if ($this->sqlite->lastErrorCode()) {
            System::log('create table error', $this->sqlite->lastErrorMsg());
            $this->sqliteConfig['ok'] = false;
            return;
        }
        $this->sqliteConfig['ok'] = true;
//        $this->sqlite->query('PRAGMA temp_store = 2');
    }

    public function workerStart($workerId)
    {
        $this->workerId = $workerId;
        $this->workerHash = 'google-earth-' . $workerId;
        //默认允许1个线程来下载数据
        if ($workerId > 0) return;
        $this->initVersion();
        $this->initSqlite();
        if (!$this->sqliteConfig['ok']) {
            System::log('init sqlite error,worker stop');
            return [];
        }
        $this->sqlite->query('begin');
        //先开启几个任务来下载数据
        $this->subWorker();
        Timer::getInstance()->loop(30000, [$this, 'noticeError']);
        while (true) {
            $this->getWork();
            Coroutine::sleep(5);
        }
    }

    public function initVersion()
    {
        $this->version = 970;
        $this->key = '';
        while (true) {
            try {
                $data = SaberGM::get('https://kh.google.com/dbRoot.v5', ['timeout' => 5]);
                $data = $data->getBody()->getContents();
                $version = substr($data, 6, 2);
                $version = current(unpack('s', $version));
                $version = $version ^ 0x4200;
                $this->version = $version;
                echo 'version,',$this->version,PHP_EOL;
                if ($this->version < 970) $this->version = 970;
                //要将前面的字符添加上
                $this->key = substr($data, 8, 1016);
                return [];
            } catch (\Throwable $throwable) {
                echo 'get version error', $throwable->getMessage(), PHP_EOL;
            }
            Coroutine::sleep(5);
        }
    }

    public function saveDbData($id, $png)
    {
        $sql = 'select id from tile where id=\'' . $id . '\'';
        $has = $this->sqlite->query($sql);
        if ($this->sqlite->lastErrorCode()) {
            echo $this->sqlite->lastErrorMsg(), PHP_EOL;
            $this->error['sum'] += 100;
            $this->error['list'][] = ['data' => ['id' => $id, 'msg' => 'check error']];
            return;
        }
        $has = $has->fetchArray(SQLITE3_ASSOC);
        if ($has['id']) {
            $this->speed['has']++;
            return;
        }
        $st = $this->sqlite->prepare('insert into tile (id,data) values (?,?)');
        $st->bindValue('1', $id, SQLITE3_TEXT);
        $st->bindValue('2', $png, SQLITE3_BLOB);
        $st->execute();
        if ($this->sqlite->lastErrorCode()) {
            $this->error['sum'] += 100;
            $this->error['list'][] = ['data' => ['id' => $id, 'msg' => 'insert error']];
            echo $this->sqlite->lastErrorMsg(), PHP_EOL;
        }
        $this->speed['ok']++;
    }

    public function getProxy($workerId = 0, $threadIndex = 0)
    {
        return true;
    }

    public function getWork()
    {
        $freeSpace = disk_free_space($this->sqliteConfig['dir']);
//        System::log('free space :' . $freeSpace / 1024 / 1024 / 1024);
        if ($freeSpace < 1024 * 1024 * 1024 * 5) {
            System::log('too disk space ' . $freeSpace / 1024 / 1024 / 1024);
            //在没有空间的时候 ，也需要上传数据，这样才将下载的数据删除掉
            $this->noticeDownloadDbs();
            Coroutine::sleep(60);
            return;
        }
        if (count($this->work['url'])) return;
        //当前已经下载完成，那么要判断是不所有的协程也下载完成了
        if ($this->thread['sum'] !== $this->thread['done']) {
            System::log('wait some thread to download ok');
            return;
        }
        //当前已经下载完成，那么需要进行将下载的数据进行提交
        if (!$this->noticeDownloadDbs()) {
            return;
        }
        //将下载的数据提交了
        if (!$this->noticeDownloadIds()) {
            return;
        }
        $speed = $this->speed['sum'] / (microtime(true) - $this->speed['time']);
        $log = ['worker id => ' . $this->workerId, ' speed=>' . $speed, 'ok=>' . $this->speed['ok'], 'has=>' . $this->speed['has'], 'sum=>' . $this->speed['sum'], 'total=>' . $this->speed['total']];
        $log = implode(' , ', $log);
        if (System::config(K::ENABLE_GOOGLE_EARTH_DOWNLOAD)) {
            //只有启用的时候，才可以打印日志
            System::log($log);
        }
        $this->sqlite->query('commit');
        if ($this->speed['total'] > $this->sqliteConfig['limit'] || ($this->work['status'] == '1' && $this->speed['total'] > 0) || (!System::config(K::ENABLE_MAP_DOWNLOAD) && $this->speed['total'] > 0)) {
            //这里有三个情况需要处理一次：
            //如果在超过了指定的长度，或者在服务器已经没有新任务，同时有下载的数据时,或者是当前不下载街景数据了，那么执行提交操作
            $this->speed['total'] = 0;
            $this->sqlite->close();
            $this->initSqlite();
            System::log('new db');
        }
        $this->sqlite->query('begin');
        $this->speed['ok'] = 0;
        $this->speed['has'] = 0;
        $this->speed['sum'] = 0;
        $this->speed['time'] = microtime(true);
        if (!System::checkSpaceOrTrafficIsEnough()) {
            System::log('traffic is not enough');
            Coroutine::sleep(60);
            return;
        }
        if (!System::config(K::ENABLE_GOOGLE_EARTH_DOWNLOAD)) {
            Coroutine::sleep(30);
            return;
        }
        if ($this->work['code']==403){
            Coroutine::sleep(60);
            return;
        }
        //先从服务器上面拉取metaData;
        $this->createMetaData();
        if (!$this->speed['ids']) return;
        //如果有metadata 数据，那么才会检查对应的区域
        if (!$this->work['work_info']){
            echo 'work info is empty', PHP_EOL;
            Coroutine::sleep(60 * 2);
            return;
        }
        if (!$this->checkWorkInfo()) {
            echo 'work info is error', PHP_EOL;
            Coroutine::sleep(60 * 5);
            return;
        }
        $parseUrl = $this->curlMetaData();
        if ($this->work['code']===403){
            Coroutine::sleep(60);
            return;
        }
        //将产生的metadata提交能主服务器
        $this->sendMetaData($parseUrl['meta']);
        //将生产的url赋值给下载列表，就可以下载了
        $this->work['url'] = $parseUrl['url'];
    }

    public function sendMetaData($meta)
    {
        $send=$meta;
        while (true) {
            $current = array_splice($send, 0, 100);
            if (!$current) break;
            try {
                //这个Workinfo 是必须要提交的；主要是使用来判断数据的正确性
                SaberGM::post(K::MANAGE_SERVER_ADDR_9502 . '/street/earth/meta', ['ids' => $current,'work_info'=>$this->work['work_info']], ['timeout' => 10]);
            } catch (\Throwable $throwable) {
                $sleep = mt_rand(2,10);
                Coroutine::sleep($sleep);
                $send=array_merge($send,$current);
                echo 'code ', $throwable->getCode(), ' send meta data error', $throwable->getMessage(), PHP_EOL;
            }
            $sleep = mt_rand(1, 2);
            Coroutine::sleep($sleep);
        }
    }

    public function createMetaData()
    {
        try {
            $data = SaberGM::get(K::MANAGE_SERVER_ADDR_9502 . '/street/earth/index', ['timeout' => 5]);
            $all = $data->getBody()->getContents();
            $all = json_decode($all, true);
            //将当前的ID要缓存起来，这样在下载完成 之后，要通知主服务器
            $this->speed['ids'] = $all['data']['ids'];
            //然后设置当前任务的级别
            if ($all['data']['work_info']) {
                $this->work['work_info']=$all['data']['work_info'];
                $this->work['start_zoom'] = $all['data']['work_info']['start_zoom'];
                $this->work['end_zoom'] = $all['data']['work_info']['end_zoom'];
                $this->work['bounds'] = $this->parseBounds($all['data']['work_info']['bounds']);
            }
        } catch (\Throwable $throwable) {
            System::log('get curl url failed ', $throwable->getMessage());
        }
    }

    public function parseBounds($bounds)
    {
        $result = ['min' => ['lng' => 999, 'lat' => 999], 'max' => ['lng' => -9999, 'lat' => -9999]];
        foreach ($bounds as $v) {
            $result['min']['lng'] = min($v['lng'], $result['min']['lng']);
            $result['min']['lat'] = min($v['lat'], $result['min']['lat']);
            $result['max']['lng'] = max($v['lng'], $result['max']['lng']);
            $result['max']['lat'] = max($v['lat'], $result['max']['lat']);
        }
        return $result;
    }

    public function curlMetaData()
    {
        $metaData = $this->speed['ids'];
        //保存当前的结果
        $parseUrl = ['url' => [], 'meta' => []];
        while (count($metaData)) {
            try {
                $current = array_shift($metaData);
                //第0级的时候，默认会返回空字符串
                if (is_null($current)) break;
                $url = str_replace(['{xyz}', '{v}'], [$current['id'], $this->version], $this->metaUrl);
                $data = SaberGM::get($url, ['timeout' => 5]);
                $data = $data->getBody()->getContents();
                $result = $this->nodeParse($data, $current['id']);
//                print_r($result);
                if ($result === false) {
                    $this->error['list'][] = ['data' => ['id'=>$current['id'],'str' => 'q2 parse error']];
                    //说明有错误，要记录下来
                } else {
                    $need = $this->getNeedData($result);
                    $parseUrl['url'] = array_merge($parseUrl['url'], $need['url']);
                    //这个是新的metaData;
                    $parseUrl['meta'] = array_merge($parseUrl['meta'], $need['meta']);
                }
            } catch (\Throwable $throwable) {
                echo $url,PHP_EOL;
                $code = $throwable->getCode();
                echo 'curl meta data error ', $throwable->getMessage(), PHP_EOL;
                switch ($code) {
                    case 0:
                    case -1:
                    case -2:
                    case -3:
                    case -4:
                    case -5:
                        $metaData[] = $current;
                        break;
                    case 403:
                        $this->work['code']=403;
                        return $parseUrl;
                        break;
                    default:
                        $this->error['list'][] = ['data' => ['id' => $current, 'str' => $throwable->getMessage(), 'code' => $code]];
//                        $this->error[]
                }
            }
        }
        return $parseUrl;
    }

    public function getNeedData($parseData)
    {
        $result = ['url' => [], 'meta' => []];
        $zoomXY = [];
        foreach ($parseData as $v) {
            if (!$v['v'] || !$v['q'] || $v['v'] == 0) {
                continue;
            }
            if (!isset($zoomXY[$v['z']])) $zoomXY[$v['z']] = $this->getBoundsXY($v['z'], 1);
            $tmpY = $v['y'] - floor(pow(2, $v['z'] + 1) / 4);
            if ($this->between($v['x'], $zoomXY[$v['z']]['start']['x'], $zoomXY[$v['z']]['end']['x']) && $this->between($tmpY, $zoomXY[$v['z']]['start']['y'], $zoomXY[$v['z']]['end']['y'])) {
                if ($v['hasSubTree'] && $v['hasImage'] && $v['z'] < $this->work['end_zoom']) {
                    //拿到符合要求的metadata;
                    $result['meta'][] = $v['q'];
                }
                if ($this->between($v['z'], $this->work['start_zoom'], $this->work['end_zoom'])) {
                    //拿到符合要求的URL
                    $result['url'][] = $v;
                }
            }
        }
        return $result;
    }

    public function getBoundsXY($z, $zoomOffset = 0)
    {
        $z += $zoomOffset;
        $start = $this->wgs84FromLatLngToPoint($this->work['bounds']['min']);
        $start['x'] = floor($start['x'] * pow(2, $z) / 256);
        $start['y'] = floor($start['y'] * pow(2, $z) / 256);
        $end = $this->wgs84FromLatLngToPoint($this->work['bounds']['max']);
        $end['x'] = floor($end['x'] * pow(2, $z) / 256);
        $end['y'] = floor($end['y'] * pow(2, $z) / 256);
        $result = ['start' => ['x' => min($start['x'], $end['x']), 'y' => min($start['y'], $end['y'])], 'end' => ['x' => max($start['x'], $end['x']), 'y' => max($start['y'], $end['y'])]];
        //由于由于经纬直投的计算方式，如果传入的纬度是-90，经度180 的时候，会多一个，这里先处理一次
        $result['end']['x'] = min(pow(2, $z) - 1, $result['end']['x']);
        if ($z < 1) $z = 1;
        $result['end']['y'] = min(pow(2, $z - 1) - 1, $result['end']['y']);
        return $result;
    }

    public function wgs84FromLatLngToPoint($latLng)
    {
        return ['x' => 256 * (0.5 + $latLng['lng'] / 360), 'y' => 128 * (90 - $latLng['lat']) / 180];
    }

    public function between($number, $start, $end, $equalEnd = true)
    {
        if ($equalEnd) {
            return $number >= $start && $number <= $end;
        } else {
            return $number >= $start && $number < $end;
        }
    }

    public function nodeParse($metadata, $quaKey = '')
    {
        $str = base64_encode($metadata);
        exec('node ' . '/node-g/node.js ' . implode('  ', [base64_encode($this->key), $str, $quaKey]), $output, $code);
//        echo 'node ' . '/node-g/node.js ' . implode('  ', [base64_encode($this->key), $str, $quaKey]),PHP_EOL;
        if ($code) {
            return false;
        }
        return json_decode($output[0], true) ?? [];
    }

    public function createTiles($id)
    {
        $r = [];
        $id = explode('$', $id);
        $zoom = current($id);
        $zoom = explode('-', $zoom);
        $zoom = end($zoom);
        $bounds = end($id);
        $bounds = json_decode($bounds, true);
        for ($i = $bounds['s']['x']; $i <= $bounds['e']['x']; $i++) {
            for ($j = $bounds['s']['y']; $j < $bounds['e']['y']; $j++) {
                $r[] = [
                    'id' => $id,
                    'tile_id' => implode('-', [$zoom, $i, $j]),
                    'url' => str_replace(['{s}', '{x}', '{y}', '{z}'], [mt_rand(0, 2), $i, $j, $zoom], $this->tileUrl),
                ];
            }
        }
        return $r;
    }

    public function curl($data)
    {
        try {
            $url = str_replace(['{xyz}', '{v}'], [$data['q'], $data['v']], $this->tileUrl);
            $result = SaberGM::get($url, ['timeout' => 10]);
            $png = $result->getBody()->getContents();
            return $this->parseImage($png);
        } catch (\Throwable $throwable) {
            $code = $throwable->getCode();
            echo 'http error code : ', $code, '   ', $throwable->getMessage(), PHP_EOL;
            switch ($code) {
                case 0:
                case -1:
                case -2:
                case -3:
                case -4:
                case -5:
                    $this->work['url'][] = $data;
                    break;
                case 403:
                    $this->work['code']=403;
                    $this->error['sum'] += 3000;
                    $data['code']=$code;
                    $data['_msg']=$throwable->getMessage();
                    $this->error['list'][] = ['data' => $data, 'code' => $code];
                    echo 'worker_id ', $this->workerId, ' === ', $throwable->getMessage(), PHP_EOL;
                    break;
                case 400:
                    $data['code']=$code;
                    $this->error['list'][] = ['data' => $data, 'code' => $code];
                    echo 'worker_id ', $this->workerId, ' === ', $throwable->getMessage(), PHP_EOL;
                    $this->error['sum']++;
                    break;
                default:
                    $data['code']=$code;
                    $this->error['list'][] = ['data' => $data, 'code' => $code];
                    echo 'worker_id ', $this->workerId, ' === ', $throwable->getMessage(), PHP_EOL;
                    $this->error['sum']++;
            }
        }
        return false;
    }

    public function subWorker()
    {
        for ($i = 0; $i < $this->thread['sum']; $i++) {
            go([$this, 'download'], $i);
        }
    }

    public function parseImage($data)
    {
        $index = 16;
        $l = strlen($data);
        $result = '';
        //在解码的时候，需要前面的8个字节
        $key=base64_decode('AAAAAAAAAAA=') . $this->key;
        for ($i = 0; $i < $l; $i++) {
            $str = ord(($data[$i])) ^ ord(($key[$index + 8]));
            $result .= chr($str);
            $index += 1;
            if ($index % 8 == 0) $index += 16;
            if ($index >= 1016) $index = ($index + 8) % 24;
        }
        return $result;
    }

    public function download($tag)
    {
        while (true) {
            if ($this->error['sum'] > $this->error['limit']) {
                echo 'too many errors', PHP_EOL;
                $this->sqlite->query('commit');
                return;
            } elseif (!$this->sqliteConfig['ok']) {
                System::log('sqlite is error,so stop download');
                return;
            } else {
                $size = count($this->work['url']);
                if ($size && $size % 400 === 0) {
                    echo 'worker id ', $this->workerId, ' count ', $size, ' error', $this->error['sum'], PHP_EOL;
                }
                $data = array_shift($this->work['url']);
                if ($data) {
                    $this->thread['status'][$tag] = false;
                    $png = $this->curl($data);
                    if ($png) {
                        //这里使用total 来表示下载了多少次，方便后面进行事务的提交
                        $this->speed['total']++;
                        $tile_id = implode([($data['z'] + 1), $data['x'], $data['y']]);
                        $this->saveDbData($tile_id, $png);
                        if ($this->speed['total'] && $this->speed['total'] % 200 === 0) {
                            $this->sqlite->query('commit');
                            $this->sqlite->query('begin');
                        }
                    }
                } else {
                    //已经没有数据了，那么要判断是不是所有的下载进程已经下载完成了
                    $this->thread['status'][$tag] = true;
                    $done = 0;
                    for ($i = 0; $i < $this->thread['sum']; $i++) {
                        if ($this->thread['status'][$i]) {
                            $done++;
                        }
                    }
                    $this->thread['done'] = $done;
                    Coroutine::sleep(1);
                }
            }
        }
    }

    private function noticeDownloadIds()
    {
        if (!$this->speed['ids']) return true;
        $result = false;
        try {
            SaberGM::post(K::MANAGE_SERVER_ADDR_9502 . '/street/earth/confirm', ['ids' => $this->speed['ids'],'work_info'=>$this->work['work_info']], ['timeout' => 5]);
            $result = true;
        } catch (\Throwable $throwable) {
            System::log('upload download ids error', $throwable->getMessage());
        }
        return $result;
    }

    public function noticeError()
    {
        if (!$this->error['list']) return;
        $post = $this->error['list'];
        try {
            //因为使用了定时器，防止在提交的过程，将新产生的数据丢弃
            $this->error['list'] = [];
            SaberGM::post(K::MANAGE_SERVER_ADDR_9502 . '/street/earth/error', $post, ['timeout' => 5]);
        } catch (\Throwable $throwable) {
            $this->error['list'] = array_merge($post, $this->error['list']);
            System::log('upload error list error ' . $throwable->getMessage());
        }
    }

    private function noticeDownloadDbs()
    {
        $result = false;
        try {
            $dbs = scandir($this->sqliteConfig['dir']);
            $dbs = array_filter($dbs, function ($v) {
                return strtolower(pathinfo($v, PATHINFO_EXTENSION)) === 'db';
            });
            $workerDb = $this->workerDbName();
            foreach ($dbs as $k => $v) {
                if ($v === $this->sqliteConfig['name'] || in_array($v, $workerDb)) unset($dbs[$k]);
            }
            $dbs = array_values($dbs);
            if (!$dbs) return true;
            $response = SaberGM::post(K::MANAGE_SERVER_ADDR_9502 . '/street/earth/db', ['dbs' => $dbs], ['timeout' => 5]);
            $str = $response->getBody()->getContents();
            $str = json_decode($str, true);
            $usedTraffic = System::config(K::STREET_TRAFFIC_USED);
            $workerDb = $this->workerDbName();
            foreach ($str['data']['del'] as $d) {
                //要删除的数据,如果是当前正在使用的，或者是别的线程在下载的，那么不能删除
                if ($d === $this->sqliteConfig['name'] || in_array($d, $workerDb)) continue;
                $usedTraffic = $usedTraffic + filesize($this->sqliteConfig['dir'] . '/' . $d);
                $un = unlink($this->sqliteConfig['dir'] . '/' . $d) ? 'success' : 'failed';
                System::log('unlink ' . $d . ' status: ' . $un);
            }
            System::config(K::STREET_TRAFFIC_USED, $usedTraffic);
            $result = true;
        } catch (\Throwable $throwable) {
            System::log('upload download dbs error', $throwable->getMessage());
        }
        return $result;
    }

    function getFenArea($fen)
    {
        if (substr($fen, 0, 1) === '-') {
            $flag = -1;
            $fen = substr($fen, 1);
        } else {
            $flag = 1;
        }
        $lng = (substr($fen, -2) - 1) * 6 - 180;
        $lat = ord((substr($fen, 0, 1))) - ord('A');
        $lat = $lat * 4;
        $bounds = [
            'min' => [
                'lng' => $lng,
            ],
            'max' => [
                'lng' => $lng + 6
            ]
        ];
        if ($flag < 0) {
            $bounds['min']['lat'] = -$lat - 4;
            $bounds['max']['lat'] = -$lat;
        } else {
            $bounds['min']['lat'] = $lat;
            $bounds['max']['lat'] = $lat + 4;
        }
        return $bounds;
    }

    public function checkWorkInfo()
    {
        $zoom = is_numeric($this->work['start_zoom']) && is_numeric($this->work['end_zoom']) && $this->work['start_zoom'] >= 0 && $this->work['end_zoom'] <= 19;
        $bounds = $this->checkBounds();
        return $zoom && $bounds&&$this->work['work_info']['id'];
    }

    public function checkBounds()
    {
        return $this->checkLng($this->work['bounds']['min']['lng']) && $this->checkLng($this->work['bounds']['max']['lng']) && $this->checkLat($this->work['bounds']['min']['lat']) && $this->checkLat($this->work['bounds']['max']['lat']);
    }

    public function checkLat($lat)
    {
        return is_numeric($lat) && abs($lat) <= 90;
    }

    public function checkLng($lng)
    {
        return is_numeric($lng) && abs($lng) <= 180;
    }
}
